==> visual-third-party-1.0.4-ibk-sources.jar: com/tenqube/visual_third/parser/loader/OneLoader.java <==
package com.tenqube.visual_third.parser.loader;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.content.LocalBroadcastManager;
import android.view.View;

import com.tenqube.visual_third.Constants;
import com.tenqube.visual_third.OnAdLoadedListener;
import com.tenqube.visual_third.entity.Category;
import com.tenqube.visual_third.entity.ClipCategory;
import com.tenqube.visual_third.manager.AdManager;
import com.tenqube.visual_third.model.api.SearchCompanyRequest;
import com.tenqube.visual_third.model.api.SearchCompanyResponse;
import com.tenqube.visual_third.model.parser.PopupData;
import com.tenqube.visual_third.model.parser.TransactionData;
import com.tenqube.visual_third.util.Mapper;

import java.util.ArrayList;

import tenqube.parser.model.ParserResult;
import tenqube.parser.model.ResultCode;
import tenqube.parser.model.SMS;
import tenqube.parser.model.Transaction;

import static com.tenqube.visual_third.Constants.ACTION_TRANSACTION_RECEIVED;
import static com.tenqube.visual_third.Constants.ACTION_TRAN_POPUP;
import static com.tenqube.visual_third.Constants.INTENT_POPUP_KEY;
import static com.tenqube.visual_third.Constants.SAMSUNG_PAY;
import static com.tenqube.visual_third.util.Utils.isEmpty;
import static com.tenqube.visual_third.util.Utils.shouldParsing;
import static tenqube.parser.core.ParserService.mIsDebug;
import static tenqube.parser.util.LogUtil.LOGI;

/**
 * SMS or MMS or NOTI 수신시 실행
 *
 */
public class OneLoader extends BaseLoader {

    private static final String TAG = OneLoader.class.getSimpleName();
    private static OneLoader mInstance = null;
    private AdManager adManager;
    private int mServerCnt = 0;

    private OneLoader(Context context) {
        super(context);
        adManager = AdManager.getInstance(context);

    }

    /**
     * ParserResult의 코드에따라서 파싱된내역 서버전송, 파싱룰 동기화 안잡힌거 전송으로 분기
     *
     * @param sms 문자 데이터
     */
    public synchronized void doParsing(final SMS sms) {

        new Thread(new Runnable() {
            @Override
            public void run() {

                LOGI(TAG, "doParsing" + sms.toString(), mIsDebug);

                // 파싱
                ParserResult parserResult = parserService.parse(sms);

                // 실패하고 삼성페이인경우 title 로 파싱
                if (parserResult.resultCode != ResultCode.SEND_TO_SERVER &&
                        SAMSUNG_PAY.equals(sms.getSender())) {

                    SMS newSms = new SMS(sms.getSmsId(),
                            sms.getTitle(),
                            sms.getSender(),
                            sms.getDisplaySender(),
                            sms.getSmsDate(),
                            sms.getSmsType(), sms.getTitle());

                    parserResult = parserService.parse(newSms);

                }

                // 파싱 결과에 따른 분기
                LOGI(TAG, "parserResult" + (parserResult.resultCode) + ResultCode.stringValueOf(parserResult.resultCode), mIsDebug);

                switch (parserResult.resultCode) {
                    case ResultCode.NEED_TO_SYNC_PARSING_RULE:
                        syncParsingRule();
                        break;

                    case ResultCode.NEED_TO_SYNC_PARSING_RULE_NO_SENDER:
                        if (shouldParsing(sms.getFullSms())) syncParsingRuleAndNoSender();
                        break;

                    case ResultCode.NEED_TO_SEND_TO_SERVER:
                        break;

                    case ResultCode.SEND_TO_SERVER:
                        parsedSMS(parserResult.transactions);
                        break;

                }

            }
        }).start();

    }

    private void syncParsingRuleAndNoSender() {
        mServerCnt++;
        if (mServerCnt == 10) {
            mServerCnt = 0;
            syncParsingRule();
        }
    }

    public static OneLoader getInstance(Context context) {
        synchronized (OneLoader.class) {
            if (mInstance == null) {
                mInstance = new OneLoader(context.getApplicationContext());
            }
        }
        return mInstance;
    }

    /**
     * 파싱된 경우
     * 결과값 저장 및 파싱된 내역 구글 캘린더 알림 노출 등 기존 로직 처리
     *
     * @param parsedTransactions 파싱된 내역
     */
    private void parsedSMS(ArrayList<Transaction> parsedTransactions) {

        try {
            ArrayList<com.tenqube.visual_third.model.parser.Transaction> transactions = Mapper.toParserTransactions(parsedTransactions);

            final com.tenqube.visual_third.model.parser.Transaction currentTran = getCurrentTransaction(transactions);

            if (currentTran == null) return;

            // 위치정보 추가
            addLocationInfo(currentTran);

            // 환율
            setCurrency(transactions);

            // 내역 합치기
            repository.mergeTransactions(transactions);

            final int tranId = repository.loadTranId(currentTran.getParsedTransaction().identifier);
            if(tranId == 0) return;

            // 기존 일괄 적용 확인 currentTran
            com.tenqube.visual_third.entity.Transaction applyAllTran = repository.loadApplyAllTran(currentTran.getParsedTransaction().keyword);

            SearchCompanyResponse searchCompanyResponse = null;

            if (applyAllTran != null) { // 서버 검색 요청 없이 사용자 내역으로 생성
                ArrayList<SearchCompanyResponse.TranCompany> results = new ArrayList<>();
                SearchCompanyResponse.Company company = new SearchCompanyResponse.Company(applyAllTran.getCompanyId(), applyAllTran.getFranchise(), "");
                SearchCompanyResponse.Category category = new SearchCompanyResponse.Category(applyAllTran.getCategoryCode() + "");
                SearchCompanyResponse.Keyword keyword = new SearchCompanyResponse.Keyword(applyAllTran.getKeyword(), applyAllTran.getSearchKeyword());
                SearchCompanyResponse.TranCompany tranCompany = new SearchCompanyResponse.TranCompany("" + applyAllTran.getIdentifier(), applyAllTran.getClassCode(), company, category, keyword);
                results.add(tranCompany);
                searchCompanyResponse = new SearchCompanyResponse(results);

            } else { // 서버 검색요청
                ArrayList<SearchCompanyRequest.Transaction> apiTransactions = Mapper.toSearchTransaction(transactions);
                if(!isEmpty(apiTransactions)) {
                    searchCompanyResponse = api.searchCompany(new SearchCompanyRequest(apiTransactions));
                }

                if(searchCompanyResponse == null) { // 기타 지출 or 수입 카테고리 할당
                    ArrayList<SearchCompanyResponse.TranCompany> results = new ArrayList<>();
                    boolean isWithdraw = currentTran.getParsedTransaction().dwType == tenqube.parser.constants.Constants.DWType.WITHDRAW.ordinal();
                    SearchCompanyResponse.Company company = new SearchCompanyResponse.Company(isWithdraw ?
                            Constants.COMPANY_ID.WITHDRAW_ETC : Constants.COMPANY_ID.DEPOSIT_ETC, "'미분류'", "");
                    SearchCompanyResponse.Category category = new SearchCompanyResponse.Category(isWithdraw ?
                            "" + Constants.CATEGORY_CODE.WITHDRAW_ETC : "" + Constants.CATEGORY_CODE.DEPOSIT_ETC);

                    SearchCompanyResponse.Keyword keyword = new SearchCompanyResponse.Keyword(currentTran.getParsedTransaction().keyword, currentTran.getParsedTransaction().keyword);
                    SearchCompanyResponse.TranCompany tranCompany = new SearchCompanyResponse.TranCompany("" + currentTran.getParsedTransaction().identifier,
                            "NF", company, category, keyword);
                    results.add(tranCompany);
                    searchCompanyResponse = new SearchCompanyResponse(results);
                }
            }

            repository.updateTransactions(searchCompanyResponse);

            // 팝업 띄움. current Tran && 중복 아닌경
            if (currentTran.getParsedTransaction().isDuplicate == 0) {
                final SearchCompanyResponse.TranCompany searchedCompany = findSearchedCompany(searchCompanyResponse, currentTran.getParsedTransaction());

                String categoryCode = currentTran.getCategoryCode();
                if(searchedCompany != null) {
                    categoryCode = searchedCompany.getCategory().getCode();
                }
                final Category category = repository.loadCategory(categoryCode);
                if(category == null) return;

                ClipCategory clipCategory = repository.loadClipCategory(categoryCode);
                if(clipCategory == null) {
                    clipCategory = new ClipCategory(0, category.getCode(), "기타", "CRDCTG100009");
                }

                final ClipCategory finalClipCategory = clipCategory;
                adManager.loadAd(new OnAdLoadedListener() {
                    @Override
                    public void onAdLoaded(View view) {
                        startTranPopup(tranId, currentTran, searchedCompany, category, finalClipCategory, view);
                    }
                });

            }
            api.syncTransactions(null);
            LocalBroadcastManager.getInstance(mContext).sendBroadcast(new Intent(ACTION_TRANSACTION_RECEIVED));

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private com.tenqube.visual_third.model.parser.Transaction getCurrentTransaction(ArrayList<com.tenqube.visual_third.model.parser.Transaction> transactions) {

        for (com.tenqube.visual_third.model.parser.Transaction transaction : transactions) {

            if (transaction.getParsedTransaction().isCurrentTran) {
                return transaction;
            }
        }
        return null;
    }

    private void addLocationInfo(com.tenqube.visual_third.model.parser.Transaction currentTran) {
        currentTran.getParsedTransaction().spentLatitude = -1;
        currentTran.getParsedTransaction().spentLongitude = -1;
    }

    /**
     * 파싱 결과 팝업
     */
    private SearchCompanyResponse.TranCompany findSearchedCompany(final SearchCompanyResponse searchCompanyResponse, Transaction currentTran) {

        SearchCompanyResponse.TranCompany searchedCompany = null;
        for (SearchCompanyResponse.TranCompany company : searchCompanyResponse.getResults()) {
            if (company.getIdentifier().equals(currentTran.identifier)) {
                searchedCompany = company;
                break;
            }
        }
        return searchedCompany;

    }

    private void startTranPopup(final int tranId,
            final com.tenqube.visual_third.model.parser.Transaction currentTran,
            final SearchCompanyResponse.TranCompany company,
            final Category category,
            final ClipCategory clipCategory,
            final View adView) {


        // 광고 호출
        new Handler(mContext.getMainLooper()).post(new Runnable() {
            public void run() {
                try {
                    TransactionData transactionData = new TransactionData(tranId, currentTran, company, category, clipCategory);

                    Bundle bundle = new Bundle();
                    bundle.putSerializable(INTENT_POPUP_KEY, new PopupData(transactionData, adView));

                    Intent intent = new Intent(ACTION_TRAN_POPUP);
                    intent.putExtra(INTENT_POPUP_KEY, bundle);

                    LocalBroadcastManager.getInstance(mContext).sendBroadcast(intent);

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

}

==> visual-third-party-1.1.2-ibk-sources.jar: com/tenqube/visual_third/parser/loader/OneLoader.java <==
package com.tenqube.visual_third.parser.loader;

import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.support.v4.content.LocalBroadcastManager;
import android.view.View;

import com.tenqube.visual_third.Constants;
import com.tenqube.visual_third.OnAdLoadedListener;
import com.tenqube.visual_third.entity.Category;
import com.tenqube.visual_third.manager.AdManager;
import com.tenqube.visual_third.model.api.SearchCompanyRequest;
import com.tenqube.visual_third.model.api.SearchCompanyResponse;
import com.tenqube.visual_third.model.ui.TransactionPopupInfo;
import com.tenqube.visual_third.ui.TransactionPopupActivity;
import com.tenqube.visual_third.util.Mapper;

import java.util.ArrayList;

import tenqube.parser.model.ParserResult;
import tenqube.parser.model.ResultCode;
import tenqube.parser.model.SMS;
import tenqube.parser.model.Transaction;

import static com.tenqube.visual_third.Constants.ACTION_TRANSACTION_RECEIVED;
import static com.tenqube.visual_third.Constants.SAMSUNG_PAY;
import static com.tenqube.visual_third.util.Utils.isEmpty;
import static com.tenqube.visual_third.util.Utils.shouldParsing;
import static tenqube.parser.core.ParserService.mIsDebug;
import static tenqube.parser.util.LogUtil.LOGI;

/**
 * SMS or MMS or NOTI 수신시 실행
 *
 */
public class OneLoader extends BaseLoader {

    private static final String TAG = OneLoader.class.getSimpleName();
    private static OneLoader mInstance = null;
    private AdManager adManager;
    private int mServerCnt = 0;

    private OneLoader(Context context) {
        super(context);
        adManager = AdManager.getInstance(context);

    }

    /**
     * ParserResult의 코드에따라서 파싱된내역 서버전송, 파싱룰 동기화 안잡힌거 전송으로 분기
     *
     * @param sms 문자 데이터
     */
    public synchronized void doParsing(final SMS sms) {

        new Thread(new Runnable() {
            @Override
            public void run() {

                LOGI(TAG, "doParsing" + sms.toString(), mIsDebug);

                // 파싱
                ParserResult parserResult = parserService.parse(sms);

                // 실패하고 삼성페이인경우 title 로 파싱
                if (parserResult.resultCode != ResultCode.SEND_TO_SERVER &&
                        SAMSUNG_PAY.equals(sms.getSender())) {

                    SMS newSms = new SMS(sms.getSmsId(),
                            sms.getTitle(),
                            sms.getSender(),
                            sms.getDisplaySender(),
                            sms.getSmsDate(),
                            sms.getSmsType(), sms.getTitle());

                    parserResult = parserService.parse(newSms);

                }

                // 파싱 결과에 따른 분기
                LOGI(TAG, "parserResult" + (parserResult.resultCode) + ResultCode.stringValueOf(parserResult.resultCode), mIsDebug);

                switch (parserResult.resultCode) {
                    case ResultCode.NEED_TO_SYNC_PARSING_RULE:
                        syncParsingRule();
                        break;

                    case ResultCode.NEED_TO_SYNC_PARSING_RULE_NO_SENDER:
                        if (shouldParsing(sms.getFullSms())) syncParsingRuleAndNoSender();
                        break;

                    case ResultCode.NEED_TO_SEND_TO_SERVER:
                        break;

                    case ResultCode.SEND_TO_SERVER:
                        parsedSMS(parserResult.transactions);
                        break;

                }

            }
        }).start();

    }

    private void syncParsingRuleAndNoSender() {
        mServerCnt++;
        if (mServerCnt == 10) {
            mServerCnt = 0;
            syncParsingRule();
        }
    }

    public static OneLoader getInstance(Context context) {
        synchronized (OneLoader.class) {
            if (mInstance == null) {
                mInstance = new OneLoader(context.getApplicationContext());
            }
        }
        return mInstance;
    }

    /**
     * 파싱된 경우
     * 결과값 저장 및 파싱된 내역 구글 캘린더 알림 노출 등 기존 로직 처리
     *
     * @param parsedTransactions 파싱된 내역
     */
    private void parsedSMS(ArrayList<Transaction> parsedTransactions) {

        try {
            ArrayList<com.tenqube.visual_third.model.parser.Transaction> transactions = Mapper.toParserTransactions(parsedTransactions);

            final com.tenqube.visual_third.model.parser.Transaction currentTran = getCurrentTransaction(transactions);

            if (currentTran == null) return;

            // 위치정보 추가
            addLocationInfo(currentTran);

            // 환율
            setCurrency(transactions);

            // 내역 합치기
            repository.mergeTransactions(transactions);

            final int tranId = repository.loadTranId(currentTran.getParsedTransaction().identifier);
            if(tranId == 0) return;

            // 기존 일괄 적용 확인 currentTran
            com.tenqube.visual_third.entity.Transaction applyAllTran = repository.loadApplyAllTran(currentTran.getParsedTransaction().keyword);

            SearchCompanyResponse searchCompanyResponse = null;

            if (applyAllTran != null) { // 서버 검색 요청 없이 사용자 내역으로 생성
                ArrayList<SearchCompanyResponse.TranCompany> results = new ArrayList<>();
                SearchCompanyResponse.Company company = new SearchCompanyResponse.Company(applyAllTran.getCompanyId(), applyAllTran.getFranchise(), "");
                SearchCompanyResponse.Category category = new SearchCompanyResponse.Category(applyAllTran.getCategoryCode() + "");
                SearchCompanyResponse.Keyword keyword = new SearchCompanyResponse.Keyword(applyAllTran.getKeyword(), applyAllTran.getSearchKeyword());
                SearchCompanyResponse.TranCompany tranCompany = new SearchCompanyResponse.TranCompany("" + applyAllTran.getIdentifier(), applyAllTran.getClassCode(), company, category, keyword);
                results.add(tranCompany);
                searchCompanyResponse = new SearchCompanyResponse(results);

            } else { // 서버 검색요청
                ArrayList<SearchCompanyRequest.Transaction> apiTransactions = Mapper.toSearchTransaction(transactions);
                if(!isEmpty(apiTransactions)) {
                    searchCompanyResponse = api.searchCompany(new SearchCompanyRequest(apiTransactions));
                }

                if(searchCompanyResponse == null) { // 기타 지출 or 수입 카테고리 할당
                    ArrayList<SearchCompanyResponse.TranCompany> results = new ArrayList<>();
                    boolean isWithdraw = currentTran.getParsedTransaction().dwType == tenqube.parser.constants.Constants.DWType.WITHDRAW.ordinal();
                    SearchCompanyResponse.Company company = new SearchCompanyResponse.Company(isWithdraw ?
                            Constants.COMPANY_ID.WITHDRAW_ETC : Constants.COMPANY_ID.DEPOSIT_ETC, "'미분류'", "");
                    SearchCompanyResponse.Category category = new SearchCompanyResponse.Category(isWithdraw ?
                            "" + Constants.CATEGORY_CODE.WITHDRAW_ETC : "" + Constants.CATEGORY_CODE.DEPOSIT_ETC);

                    SearchCompanyResponse.Keyword keyword = new SearchCompanyResponse.Keyword(currentTran.getParsedTransaction().keyword, currentTran.getParsedTransaction().keyword);
                    SearchCompanyResponse.TranCompany tranCompany = new SearchCompanyResponse.TranCompany("" + currentTran.getParsedTransaction().identifier,
                            "NF", company, category, keyword);
                    results.add(tranCompany);
                    searchCompanyResponse = new SearchCompanyResponse(results);
                }
            }

            repository.updateTransactions(searchCompanyResponse);

            // 팝업 띄움. current Tran && 중복 아닌경
            if (currentTran.getParsedTransaction().isDuplicate == 0) {
                final SearchCompanyResponse.TranCompany searchedCompany = findSearchedCompany(searchCompanyResponse, currentTran.getParsedTransaction());

                String categoryCode = currentTran.getCategoryCode();
                if(searchedCompany != null) {
                    categoryCode = searchedCompany.getCategory().getCode();
                }
                final Category category = repository.loadCategory(categoryCode);
                if(category == null) return;

                adManager.loadAd(new OnAdLoadedListener() {
                    @Override
                    public void onAdLoaded(View view) {
                        startTranPopup(tranId, currentTran, searchedCompany, category);
                    }
                });

            }
            api.syncTransactions(null);
            LocalBroadcastManager.getInstance(mContext).sendBroadcast(new Intent(ACTION_TRANSACTION_RECEIVED));

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private com.tenqube.visual_third.model.parser.Transaction getCurrentTransaction(ArrayList<com.tenqube.visual_third.model.parser.Transaction> transactions) {

        for (com.tenqube.visual_third.model.parser.Transaction transaction : transactions) {

            if (transaction.getParsedTransaction().isCurrentTran) {
                return transaction;
            }
        }
        return null;
    }

    private void addLocationInfo(com.tenqube.visual_third.model.parser.Transaction currentTran) {
        currentTran.getParsedTransaction().spentLatitude = -1;
        currentTran.getParsedTransaction().spentLongitude = -1;
    }

    /**
     * 파싱 결과 팝업
     */
    private SearchCompanyResponse.TranCompany findSearchedCompany(final SearchCompanyResponse searchCompanyResponse, Transaction currentTran) {

        SearchCompanyResponse.TranCompany searchedCompany = null;
        for (SearchCompanyResponse.TranCompany company : searchCompanyResponse.getResults()) {
            if (company.getIdentifier().equals(currentTran.identifier)) {
                searchedCompany = company;
                break;
            }
        }
        return searchedCompany;

    }

    private void startTranPopup(final int tranId,
            final com.tenqube.visual_third.model.parser.Transaction currentTran,
            final SearchCompanyResponse.TranCompany company,
            final Category category) {


        // 광고 호출
        new Handler(mContext.getMainLooper()).post(new Runnable() {
            public void run() {
                try {
                    TransactionPopupInfo transactionPopupInfo = new TransactionPopupInfo(tranId, currentTran, company, category);

                    TransactionPopupActivity.startActivity(mContext, transactionPopupInfo);

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

}
